@@ -18,8 +18,8 @@
 #include <simgear_config.h>
 #include <simgear/package/Install.hxx>
 
-#include <boost/foreach.hpp>
 #include <fstream>
+#include <cstdlib>
 
 #include <simgear/package/unzip.h>
 #include <simgear/package/md5.h>
@@ -36,10 +36,6 @@
 #include <simgear/misc/strutils.hxx>
 #include <simgear/io/iostreams/sgstream.hxx>
 
-extern "C" {
-    void fill_memory_filefunc (zlib_filefunc_def*);
-}
-
 namespace simgear {
 
 namespace pkg {
@@ -47,26 +43,21 @@ namespace pkg {
 class Install::PackageArchiveDownloader : public HTTP::Request
 {
 public:
-    PackageArchiveDownloader(InstallRef aOwner) :
+    PackageArchiveDownloader(InstallRef aOwner, const string_list& urls) :
         HTTP::Request("" /* dummy URL */),
-        m_owner(aOwner),
-        m_downloaded(0)
+        m_owner(aOwner)
     {
-        m_urls = m_owner->package()->downloadUrls();
-        if (m_urls.empty()) {
-            throw sg_exception("no package download URLs");
+        m_urls = urls;
+        if (urls.empty()) {
+            m_urls = m_owner->package()->downloadUrls();
         }
 
-        if (m_owner->package()->properties()->hasChild("archive-type")) {
-            setArchiveTypeFromExtension(m_owner->package()->properties()->getStringValue("archive-type"));
-        }
+        selectMirrorUrl();
         
-        // TODO randomise order of m_urls
-
         m_extractPath = aOwner->path().dir();
         m_extractPath.append("_extract_" + aOwner->package()->md5());
 
-        // clean up any existing files
+        // clean up any existing files (eg from previous failed download)
         Dir d(m_extractPath);
         if (d.exists()) {
             d.remove(true /* recursive */);
@@ -98,35 +89,54 @@ public:
         return (m_downloaded * 100) / responseLength();
     }
 protected:
-    virtual std::string url() const
+    void selectMirrorUrl()
     {
-        return m_urls.front();
+        const int randomizedIndex = rand() % m_urls.size();
+        m_activeURL = m_urls.at(randomizedIndex);
+        m_urls.erase(m_urls.begin() + randomizedIndex);
+    }
+    
+    virtual std::string url() const override
+    {
+        return m_activeURL;
     }
 
-    virtual void responseHeadersComplete()
+    void responseHeadersComplete() override
     {
         Request::responseHeadersComplete();
 
         Dir d(m_extractPath);
-        d.create(0755);
+        if (!d.create(0755)) {
+            SG_LOG(SG_GENERAL, SG_WARN, "Failed to create extraction directory" << d.path());
+        }
 
+		m_extractor.reset(new ArchiveExtractor(m_extractPath));
         memset(&m_md5, 0, sizeof(SG_MD5_CTX));
         SG_MD5Init(&m_md5);
     }
 
-    virtual void gotBodyData(const char* s, int n)
+    void gotBodyData(const char* s, int n) override
     {
-        m_buffer += std::string(s, n);
-        SG_MD5Update(&m_md5, (unsigned char*) s, n);
-
-        m_downloaded = m_buffer.size();
-        m_owner->installProgress(m_buffer.size(), responseLength());
+        // if there's a pre-existing error, discard byte sinstead of pushing
+        // more through the extactor
+        if (m_extractor->hasError()) {
+            return;
+        }
+        
+		const uint8_t* ubytes = (uint8_t*) s;
+        SG_MD5Update(&m_md5, ubytes, n);
+        m_downloaded += n;
+        m_owner->installProgress(m_downloaded, responseLength());
+		m_extractor->extractBytes(ubytes, n);
+        if (m_extractor->hasError()) {
+            SG_LOG(SG_GENERAL, SG_WARN, "archive extraction failed (from " + m_activeURL + ")");
+        }
     }
 
-    virtual void onDone()
+    void onDone() override
     {
         if (responseCode() != 200) {
-            SG_LOG(SG_GENERAL, SG_ALERT, "download failure:" << responseCode() <<
+            SG_LOG(SG_GENERAL, SG_WARN, "download failure:" << responseCode() <<
                    "\n\t" << url());
             Delegate::StatusCode code = Delegate::FAIL_DOWNLOAD;
             if (responseCode() == 404) {
@@ -143,7 +153,7 @@ protected:
           strutils::encodeHex(digest, MD5_DIGEST_LENGTH);
 
         if (hex_md5 != m_owner->package()->md5()) {
-            SG_LOG(SG_GENERAL, SG_ALERT, "md5 verification failed:\n"
+            SG_LOG(SG_GENERAL, SG_WARN, "md5 verification failed:\n"
                 << "\t" << hex_md5 << "\n\t"
                 << m_owner->package()->md5() << "\n\t"
                 << "downloading from:" << url());
@@ -151,7 +161,8 @@ protected:
             return;
         }
 
-        if (!extract()) {
+		m_extractor->flush();
+        if (m_extractor->hasError() || !m_extractor->isAtEndOfArchive()) {
             SG_LOG(SG_GENERAL, SG_WARN, "archive extraction failed");
             doFailure(Delegate::FAIL_EXTRACT);
             return;
@@ -197,7 +208,7 @@ protected:
         m_owner->installResult(Delegate::STATUS_SUCCESS);
     }
 
-    virtual void onFail()
+    void onFail() override
     {
         if (responseCode() == -1) {
             doFailure(Delegate::USER_CANCELLED);
@@ -207,176 +218,39 @@ protected:
     }
 
 private:
-    void setArchiveTypeFromExtension(const std::string& ext)
-    {
-        if (ext.empty())
-            return;
-        
-        if (ext == "zip") {
-            m_archiveType = ZIP;
-            return;
-        }
-        
-        if ((ext == "tar.gz") || (ext == "tgz")) {
-            m_archiveType = TAR_GZ;
-            return;
-        }
-    }
-    
-    void extractCurrentFile(unzFile zip, char* buffer, size_t bufferSize)
-    {
-        unz_file_info fileInfo;
-        unzGetCurrentFileInfo(zip, &fileInfo,
-            buffer, bufferSize,
-            NULL, 0,  /* extra field */
-            NULL, 0 /* comment field */);
-
-        std::string name(buffer);
-    // no absolute paths, no 'up' traversals
-    // we could also look for suspicious file extensions here (forbid .dll, .exe, .so)
-        if ((name[0] == '/') || (name.find("../") != std::string::npos) || (name.find("..\\") != std::string::npos)) {
-            throw sg_format_exception("Bad zip path", name);
-        }
-
-        if (fileInfo.uncompressed_size == 0) {
-            // assume it's a directory for now
-            // since we create parent directories when extracting
-            // a path, we're done here
-            return;
-        }
-
-        int result = unzOpenCurrentFile(zip);
-        if (result != UNZ_OK) {
-            throw sg_io_exception("opening current zip file failed", sg_location(name));
-        }
-
-        sg_ofstream outFile;
-        bool eof = false;
-        SGPath path(m_extractPath);
-        path.append(name);
-
-    // create enclosing directory heirarchy as required
-        Dir parentDir(path.dir());
-        if (!parentDir.exists()) {
-            bool ok = parentDir.create(0755);
-            if (!ok) {
-                throw sg_io_exception("failed to create directory heirarchy for extraction", path);
-            }
-        }
-
-        outFile.open(path, std::ios::binary | std::ios::trunc | std::ios::out);
-        if (outFile.fail()) {
-            throw sg_io_exception("failed to open output file for writing", path);
-        }
-
-        while (!eof) {
-            int bytes = unzReadCurrentFile(zip, buffer, bufferSize);
-            if (bytes < 0) {
-                throw sg_io_exception("unzip failure reading curent archive", sg_location(name));
-            } else if (bytes == 0) {
-                eof = true;
-            } else {
-                outFile.write(buffer, bytes);
-            }
-        }
-
-        outFile.close();
-        unzCloseCurrentFile(zip);
-    }
-
-    bool extract()
-    {
-        const std::string u(url());
-        const size_t ul(u.length());
-        
-        if (m_archiveType == AUTO_DETECT) {
-            if (u.rfind(".zip") == (ul - 4)) {
-                m_archiveType = ZIP;
-            } else if (u.rfind(".tar.gz") == (ul - 7)) {
-                m_archiveType = TAR_GZ;
-            }
-            // we will fall through to the error case now
-        }
-        
-        if (m_archiveType == ZIP) {
-            return extractUnzip();
-        } else if (m_archiveType == TAR_GZ) {
-            return extractTar();
-        }
-        
-        SG_LOG(SG_IO, SG_WARN, "unsupported archive format:" << u);
-        return false;
-    }
-
-    bool extractUnzip()
-    {
-        bool result = true;
-        zlib_filefunc_def memoryAccessFuncs;
-        fill_memory_filefunc(&memoryAccessFuncs);
-
-        char bufferName[128];
-        snprintf(bufferName, 128, "%p+%lx", m_buffer.data(), m_buffer.size());
-        unzFile zip = unzOpen2(bufferName, &memoryAccessFuncs);
-
-        const size_t BUFFER_SIZE = 32 * 1024;
-        void* buf = malloc(BUFFER_SIZE);
-
-        try {
-            int result = unzGoToFirstFile(zip);
-            if (result != UNZ_OK) {
-                throw sg_exception("failed to go to first file in archive");
-            }
-
-            while (true) {
-                extractCurrentFile(zip, (char*) buf, BUFFER_SIZE);
-                result = unzGoToNextFile(zip);
-                if (result == UNZ_END_OF_LIST_OF_FILE) {
-                    break;
-                } else if (result != UNZ_OK) {
-                    throw sg_io_exception("failed to go to next file in the archive");
-                }
-            }
-        } catch (sg_exception& ) {
-            result = false;
-        }
-
-        free(buf);
-        unzClose(zip);
-        return result;
-    }
-
-    bool extractTar()
-    {
-        TarExtractor tx(m_extractPath);
-        tx.extractBytes(m_buffer.data(), m_buffer.size());
-        return !tx.hasError() && tx.isAtEndOfArchive();
-    }
-
     void doFailure(Delegate::StatusCode aReason)
     {
         Dir dir(m_extractPath);
         if (dir.exists()) {
             dir.remove(true /* recursive */);
         }
+        
+        const auto canRetry = (aReason == Delegate::FAIL_NOT_FOUND) ||
+            (aReason == Delegate::FAIL_DOWNLOAD) || (aReason == Delegate::FAIL_CHECKSUM);
+        
+        if (canRetry && !m_urls.empty()) {
+            SG_LOG(SG_GENERAL, SG_WARN, "archive download failed from:" << m_activeURL
+                   << "\n\twill retry with next mirror");
+            // becuase selectMirrorUrl erased the active URL from m_urls,
+            // this new request will select one of the other mirrors
+            auto retryDownload = new PackageArchiveDownloader(m_owner, m_urls);
+            m_owner->m_download.reset(retryDownload);
+            m_owner->package()->catalog()->root()->makeHTTPRequest(retryDownload);
+            return;
+        }
 
-        // TODO - try other mirrors
         m_owner->m_download.reset(); // ensure we get cleaned up
         m_owner->installResult(aReason);
     }
 
-    enum ArchiveType {
-        AUTO_DETECT = 0,
-        ZIP,
-        TAR_GZ
-    };
     
     InstallRef m_owner;
-    ArchiveType m_archiveType = AUTO_DETECT;
+    string m_activeURL;
     string_list m_urls;
     SG_MD5_CTX m_md5;
-    std::string m_buffer;
     SGPath m_extractPath;
-    size_t m_downloaded;
+    size_t m_downloaded = 0;
+	std::unique_ptr<ArchiveExtractor> m_extractor;
 };
 
 ////////////////////////////////////////////////////////////////////
@@ -435,7 +309,7 @@ void Install::startUpdate()
         return; // already active
     }
 
-    m_download = new PackageArchiveDownloader(this);
+    m_download = new PackageArchiveDownloader(this, {});
     m_package->catalog()->root()->makeHTTPRequest(m_download);
     m_package->catalog()->root()->startInstall(this);
 }
@@ -495,6 +369,7 @@ void Install::cancelDownload()
     }
 
     m_package->catalog()->root()->cancelDownload(this);
+    m_download.clear();
 }
 
 SGPath Install::primarySetPath() const
