@@ -1,4 +1,4 @@
-// tileentry.cxx -- routines to handle a scenery tile
+// ReaderWriterSTG.cxx -- routines to handle a scenery tile
 //
 // Written by Curtis Olson, started May 1998.
 //
@@ -42,7 +42,7 @@
 #include <simgear/math/SGGeometry.hxx>
 #include <simgear/bucket/newbucket.hxx>
 #include <simgear/debug/logstream.hxx>
-#include <simgear/misc/sgstream.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/scene/util/OptionsReadFileCallback.hxx>
 #include <simgear/scene/util/OsgMath.hxx>
 #include <simgear/scene/util/QuadTreeBuilder.hxx>
@@ -53,6 +53,13 @@
 
 #include "SGOceanTile.hxx"
 
+#define BUILDING_ROUGH "OBJECT_BUILDING_MESH_ROUGH"
+#define BUILDING_DETAILED "OBJECT_BUILDING_MESH_DETAILED"
+#define ROAD_ROUGH "OBJECT_ROAD_ROUGH"
+#define ROAD_DETAILED "OBJECT_ROAD_DETAILED"
+#define RAILWAY_ROUGH "OBJECT_RAILWAY_ROUGH"
+#define RAILWAY_DETAILED "OBJECT_RAILWAY_DETAILED"
+
 namespace simgear {
 
 /// Ok, this is a hack - we do not exactly know if it's an airport or not.
@@ -87,14 +94,14 @@ static SGBucket bucketIndexFromFileName(const std::string& fileName)
 
 struct ReaderWriterSTG::_ModelBin {
     struct _Object {
-        std::string _errorLocation;
+        SGPath _errorLocation;
         std::string _token;
         std::string _name;
         osg::ref_ptr<SGReaderWriterOptions> _options;
     };
     struct _ObjectStatic {
         _ObjectStatic() : _agl(false), _proxy(false), _lon(0), _lat(0), _elev(0), _hdg(0), _pitch(0), _roll(0), _range(SG_OBJECT_RANGE_ROUGH) { }
-        std::string _errorLocation;
+        SGPath _errorLocation;
         std::string _token;
         std::string _name;
         bool _agl;
@@ -106,7 +113,7 @@ struct ReaderWriterSTG::_ModelBin {
     };
     struct _Sign {
         _Sign() : _agl(false), _lon(0), _lat(0), _elev(0), _hdg(0), _size(-1) { }
-        std::string _errorLocation;
+        SGPath _errorLocation;
         std::string _token;
         std::string _name;
         bool _agl;
@@ -212,7 +219,6 @@ struct ReaderWriterSTG::_ModelBin {
         _object_range_bare(SG_OBJECT_RANGE_BARE),
         _object_range_rough(SG_OBJECT_RANGE_ROUGH),
         _object_range_detailed(SG_OBJECT_RANGE_DETAILED),
-        _building_mesh_enabled(false),
         _foundBase(false)
     { }
 
@@ -284,29 +290,42 @@ struct ReaderWriterSTG::_ModelBin {
         return SGGeod::fromCart(cart).getElevationM();
     }
 
-    bool read(const std::string& absoluteFileName, const osgDB::Options* options)
-    {
-				// Determine object ranges.  Mesh size of 2000mx2000m needs to be accounted for.
-				_object_range_bare = 1414.0f + atof(options->getPluginStringData("SimGear::LOD_RANGE_BARE").c_str());
-				_object_range_rough = 1414.0f + atof(options->getPluginStringData("SimGear::LOD_RANGE_ROUGH").c_str());
-				_object_range_detailed = 1414.0f + atof(options->getPluginStringData("SimGear::LOD_RANGE_DETAILED").c_str());
-				_building_mesh_enabled = (options->getPluginStringData("SimGear::RENDER_BUILDING_MESH") == "true");
+    void checkInsideBucket(const SGPath& absoluteFileName, float lon, float lat) {
+        SGBucket bucket = bucketIndexFromFileName(absoluteFileName.file_base().c_str());
+        SGBucket correctBucket = SGBucket( SGGeod::fromDeg(lon, lat));
 
-        if (absoluteFileName.empty())
+        if (bucket != correctBucket) {
+          SG_LOG( SG_TERRAIN, SG_DEV_WARN, absoluteFileName
+                  << ": Object at " << lon << ", " << lat <<
+                  " in incorrect bucket (" << bucket << ") - should be in " <<
+                  correctBucket.gen_index_str() << " (" << correctBucket << ")");
+        }
+    }
+
+    bool read(const SGPath& absoluteFileName, const osgDB::Options* options)
+    {
+        if (!absoluteFileName.exists()) {
             return false;
+        }
 
         sg_gzifstream stream(absoluteFileName);
-        if (!stream.is_open())
+        if (!stream.is_open()) {
             return false;
+        }
+
+        // Determine object ranges.  Mesh size of 2000mx2000m needs to be accounted for.
+        _object_range_bare = 1414.0f + atof(options->getPluginStringData("SimGear::LOD_RANGE_BARE").c_str());
+        _object_range_rough = 1414.0f + atof(options->getPluginStringData("SimGear::LOD_RANGE_ROUGH").c_str());
+        _object_range_detailed = 1414.0f + atof(options->getPluginStringData("SimGear::LOD_RANGE_DETAILED").c_str());
 
         SG_LOG(SG_TERRAIN, SG_INFO, "Loading stg file " << absoluteFileName);
 
-        std::string filePath = osgDB::getFilePath(absoluteFileName);
+        std::string filePath = osgDB::getFilePath(absoluteFileName.local8BitStr());
 
         // Bucket provides a consistent seed
         // so we have consistent set of pseudo-random numbers for each STG file
         mt seed;
-        int bucket = atoi(SGPath(absoluteFileName).file_base().c_str());
+        int bucket = atoi(absoluteFileName.file_base().c_str());
         mt_init(&seed, bucket);
 
         // do only load airport btg files.
@@ -381,6 +400,7 @@ struct ReaderWriterSTG::_ModelBin {
 										else
 												opt->setInstantiateEffects(false);
 										_ObjectStatic obj;
+
 										obj._errorLocation = absoluteFileName;
 										obj._token = token;
 										obj._name = name;
@@ -389,7 +409,8 @@ struct ReaderWriterSTG::_ModelBin {
 										in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg >> obj._pitch >> obj._roll;
 										obj._range = range;
 										obj._options = opt;
-										_objectStaticList.push_back(obj);
+                    checkInsideBucket(absoluteFileName, obj._lon, obj._lat);
+                    _objectStaticList.push_back(obj);
                 } else if (token == "OBJECT_SHARED" || token == "OBJECT_SHARED_AGL") {
 										osg::ref_ptr<SGReaderWriterOptions> opt;
 										opt = sharedOptions(filePath, options);
@@ -406,7 +427,8 @@ struct ReaderWriterSTG::_ModelBin {
 										in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg >> obj._pitch >> obj._roll;
 										obj._range = range;
 										obj._options = opt;
-										_objectStaticList.push_back(obj);
+                    checkInsideBucket(absoluteFileName, obj._lon, obj._lat);
+                    _objectStaticList.push_back(obj);
                 } else if (token == "OBJECT_SIGN" || token == "OBJECT_SIGN_AGL") {
 										_Sign sign;
 										sign._token = token;
@@ -414,43 +436,57 @@ struct ReaderWriterSTG::_ModelBin {
 										sign._agl = (token == "OBJECT_SIGN_AGL");
 										in >> sign._lon >> sign._lat >> sign._elev >> sign._hdg >> sign._size;
 										_signList.push_back(sign);
-                } else if (token == "OBJECT_BUILDING_MESH_ROUGH" || token == "OBJECT_BUILDING_MESH_DETAILED") {
+                } else if (token == BUILDING_ROUGH || token == BUILDING_DETAILED ||
+                           token == ROAD_ROUGH     || token == ROAD_DETAILED     ||
+                           token == RAILWAY_ROUGH  || token == RAILWAY_DETAILED)   {
+                    osg::ref_ptr<SGReaderWriterOptions> opt;
+                    opt = staticOptions(filePath, options);
+                    _ObjectStatic obj;
 
-										// Only load if building mesh enabled to avoid impacting low-powered systems
-										if (_building_mesh_enabled) {
-											osg::ref_ptr<SGReaderWriterOptions> opt;
-											opt = staticOptions(filePath, options);
-											if (SGPath(name).lower_extension() == "ac")
-													opt->setInstantiateEffects(true);
-											else
-													opt->setInstantiateEffects(false);
-											_ObjectStatic obj;
-											obj._errorLocation = absoluteFileName;
-											obj._token = token;
-											obj._name = name;
-											obj._agl = false;
-											obj._proxy = true;
-											in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg >> obj._pitch >> obj._roll;
+                    opt->setInstantiateEffects(false);
+                    if (SGPath(name).lower_extension() == "ac") {
+                      // Generate material/Effects lookups for raw models, i.e.
+                      // those not wrapped in an XML while will include Effects
+                      opt->setInstantiateMaterialEffects(true);
 
-											if (token == "OBJECT_BUILDING_MESH_DETAILED") {
-												// Apply a lower LOD range if this is a detailed building
-												range = _object_range_detailed;
-												double lrand = mt_rand(&seed);
-												if      (lrand < 0.1) range = range * 2.0;
-												else if (lrand < 0.4) range = range * 1.5;
-											}
-
-											obj._range = range;
-											obj._options = opt;
-											_objectStaticList.push_back(obj);
+                      if (token == BUILDING_ROUGH || token == BUILDING_DETAILED) {
+                        opt->setMaterialName("OSM_Building");
+                      } else if (token == ROAD_ROUGH || token == ROAD_DETAILED) {
+                        opt->setMaterialName("OSM_Road");
+                      } else if (token == RAILWAY_ROUGH || token == RAILWAY_DETAILED) {
+                        opt->setMaterialName("OSM_Railway");
+                      } else {
+                        // Programming error.  If we get here then someone has added a verb to the list of
+                        // tokens above but not in this set of if-else statements.
+                        SG_LOG(SG_TERRAIN, SG_ALERT, "Programming Error - STG token without material branch");
+                      }
                     }
+
+                    obj._errorLocation = absoluteFileName;
+                    obj._token = token;
+                    obj._name = name;
+                    obj._agl = false;
+                    obj._proxy = true;
+                    in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg >> obj._pitch >> obj._roll;
+
+                    opt->setLocation(obj._lon, obj._lat);
+                    if (token == BUILDING_DETAILED || token == ROAD_DETAILED || token == RAILWAY_DETAILED ) {
+											// Apply a lower LOD range if this is a detailed mesh
+											range = _object_range_detailed;
+											double lrand = mt_rand(&seed);
+											if      (lrand < 0.1) range = range * 2.0;
+											else if (lrand < 0.4) range = range * 1.5;
+										}
+
+                    obj._range = range;
+
+                    obj._options = opt;
+                    checkInsideBucket(absoluteFileName, obj._lon, obj._lat);
+                    _objectStaticList.push_back(obj);
                 } else {
                     SG_LOG( SG_TERRAIN, SG_ALERT, absoluteFileName
                             << ": Unknown token '" << token << "'" );
                 }
-            } else {
-                SG_LOG( SG_TERRAIN, SG_ALERT, absoluteFileName
-                        << ": Unknown token '" << token << "'" );
             }
         }
 
@@ -467,12 +503,12 @@ struct ReaderWriterSTG::_ModelBin {
         terrainGroup->setName("terrain");
 
         if (_foundBase) {
-            for (std::list<_Object>::iterator i = _objectList.begin(); i != _objectList.end(); ++i) {
+            for (auto stgObject : _objectList) {
                 osg::ref_ptr<osg::Node> node;
-                node = osgDB::readRefNodeFile(i->_name, i->_options.get());
+                node = osgDB::readRefNodeFile(stgObject._name, stgObject._options.get());
                 if (!node.valid()) {
-                    SG_LOG(SG_TERRAIN, SG_ALERT, i->_errorLocation << ": Failed to load "
-                           << i->_token << " '" << i->_name << "'");
+                    SG_LOG(SG_TERRAIN, SG_ALERT, stgObject._errorLocation << ": Failed to load "
+                           << stgObject._token << " '" << stgObject._name << "'");
                     continue;
                 }
                 terrainGroup->addChild(node.get());
@@ -528,7 +564,8 @@ struct ReaderWriterSTG::_ModelBin {
 
             // Objects may end up displayed up to 2x the object range.
             pagedLOD->setRange(pagedLOD->getNumChildren(), 0, 2.0 * _object_range_rough + SG_TILE_RADIUS);
-
+            SG_LOG( SG_TERRAIN, SG_DEBUG, "Tile PagedLOD Center: " << pagedLOD->getCenter().x() << "," << pagedLOD->getCenter().y() << "," << pagedLOD->getCenter().z() );
+            SG_LOG( SG_TERRAIN, SG_DEBUG, "Tile PagedLOD Range: " << (2.0 * _object_range_rough + SG_TILE_RADIUS));
             return pagedLOD;
         }
     }
@@ -536,7 +573,6 @@ struct ReaderWriterSTG::_ModelBin {
     double _object_range_bare;
     double _object_range_rough;
     double _object_range_detailed;
-    bool _building_mesh_enabled;
     bool _foundBase;
     std::list<_Object> _objectList;
     std::list<_ObjectStatic> _objectStaticList;
@@ -571,31 +607,44 @@ ReaderWriterSTG::readNode(const std::string& fileName, const osgDB::Options* opt
     if (osgDB::getSimpleFileName(fileName) != fileName) {
         if (!modelBin.read(fileName, options))
             return ReadResult::FILE_NOT_FOUND;
-    } else {
-        // For stg meta files, we need options for the search path.
-        if (!options)
-            return ReadResult::FILE_NOT_FOUND;
+    }
 
-        SG_LOG(SG_TERRAIN, SG_INFO, "Loading tile " << fileName);
+    // For stg meta files, we need options for the search path.
+    if (!options) {
+        return ReadResult::FILE_NOT_FOUND;
+    }
 
-        std::string basePath = bucket.gen_base_path();
+    osg::ref_ptr<SGReaderWriterOptions> sgOpts(SGReaderWriterOptions::copyOrCreate(options));
+    if (sgOpts->getSceneryPathSuffixes().empty()) {
+        SG_LOG(SG_TERRAIN, SG_ALERT, "Loading tile " << fileName << ", no scenery path suffixes were configured so giving up");
+        return ReadResult::FILE_NOT_FOUND;
 
-        // Stop scanning once an object base is found
-        // This is considered a meta file, so apply the scenery path search
-        const osgDB::FilePathList& filePathList = options->getDatabasePathList();
-        for (osgDB::FilePathList::const_iterator i = filePathList.begin();
-             i != filePathList.end() && !modelBin._foundBase; ++i) {
-            SGPath objects(*i);
-            objects.append("Objects");
-            objects.append(basePath);
-            objects.append(fileName);
-            modelBin.read(objects.local8BitStr(), options);
+    }
 
-            SGPath terrain(*i);
-            terrain.append("Terrain");
-            terrain.append(basePath);
-            terrain.append(fileName);
-            modelBin.read(terrain.local8BitStr(), options);
+    SG_LOG(SG_TERRAIN, SG_INFO, "Loading tile " << fileName);
+
+    std::string basePath = bucket.gen_base_path();
+
+    // Stop scanning paths once an object base is found
+    // But do load all STGs at the same level (i.e from the same scenery path)
+    const osgDB::FilePathList& filePathList = options->getDatabasePathList();
+    for (auto path : filePathList) {
+        if (modelBin._foundBase) {
+            break;
+        }
+
+        SGPath base(path);
+        // check for non-suffixed file, and warn.
+        SGPath pathWithoutSuffix = base / basePath / fileName;
+        if (pathWithoutSuffix.exists()) {
+            SG_LOG(SG_TERRAIN, SG_ALERT, "Found scenery file " << pathWithoutSuffix << " in scenery path " << path
+                   << ".\nScenery paths without type subdirectories are no longer supported, please move thse files\n"
+                   << "into a an appropriate subdirectory, for example:" << base / "Objects" / basePath / fileName);
+        }
+
+        for (auto suffix : sgOpts->getSceneryPathSuffixes()) {
+            SGPath p = base / suffix / basePath / fileName;
+            modelBin.read(p, options);
         }
     }
 
